<?php
/**
 * @category    Klaviyo
 * @package     Klaviyo_Reclaim
 * @copyright   Copyright (c) 2013 Klaviyo Inc. (http://www.klaviyo.com)
 */


/**
 * Reclaim Tracker
 *
 * @category   Klaviyo
 * @package    Klaviyo_Reclaim
 * @author     Klaviyo Team <support@klaviyo.com>
 */

class Klaviyo_Reclaim_Model_Tracker {
    public $api_key;
    public $api_revision = '2023-08-15';
    public $host = 'https://a.klaviyo.com/client/%s/?company_id=%s';

    protected $TRACK_ONCE_KEY = '__track_once__';

    public function __construct($api_key) {
        $this->api_key = $api_key;
    }

    function track($event, $customer_properties=array(), $properties=array(), $timestamp=NULL) {
        if ((!array_key_exists('$email', $customer_properties) || empty($customer_properties['$email']))
            && (!array_key_exists('$id', $customer_properties) || empty($customer_properties['$id']))) {

            throw new Exception('You must identify a user by email or ID.');
        }

        $body = $this->build_event_properties($properties, $customer_properties, $event, $timestamp);
        return $this->make_request('events', $body);
    }

    function track_once($event, $customer_properties=array(), $properties=array(), $timestamp=NULL) {
        $properties[$this->TRACK_ONCE_KEY] = true;
        return $this->track($event, $customer_properties, $properties, $timestamp);
    }

    function identify($properties) {
        if ((!array_key_exists('$email', $properties) || empty($properties['$email']))
            && (!array_key_exists('$id', $properties) || empty($properties['$id']))) {

            throw new Exception('You must identify a user by email or ID.');
        }

        $body = $this->build_customer_properties($properties);
        return $this->make_request('profiles', $body);
    }

    protected function build_event_properties($event_properties, $customer_properties, $event, $timestamp) {
        $timestamp = new DateTime(sprintf("@%s", $timestamp));

        return array(
            'data' => array(
                'type' => 'event',
                'attributes' => array(
                    'properties' => $event_properties,
                    'value' => $event_properties['$value'],
                    'time' => $timestamp->format("c"),
                    'metric' => array(
                        'data' => array(
                            'type' => 'metric',
                            'attributes' => array(
                                'name' => $event,
                                'service' => 'magento',
                            )
                        )
                    ),
                    'profile' => $this->build_customer_properties($customer_properties)
                )
            )
        );
    }

    protected function build_customer_properties($customer_properties) {
        $email = '';
        $id = '';

        if (isset($customer_properties['$email'])) {
            $email = $customer_properties['$email'];
            unset($customer_properties['$email']);
        }

        if (isset($customer_properties['$id'])) {
            $id = $customer_properties['$id'];
            unset($customer_properties['$id']);
        }

        return array(
            'data' => array(
                'type' => 'profile',
                'id' => $id,
                'attributes' => array(
                    'email' => $email,
                    'properties' => $customer_properties
                )
            )
        );
    }

    protected function make_request($path, $body) {
        $url = sprintf($this->host, $path, $this->api_key);
        // Try and use the zend framework to make the request to Klaviyo
        try {
            $client = new Zend_Http_Client($url);
            $client->setMethod('POST');

            $client->setHeaders(array(
                "Content-Type" => "application/json",
                "Accept" => "application/json",
                "revision" => $this->api_revision,
                "X-Klaviyo-User-Agent" => 'magento-klaviyo/' . Klaviyo_Reclaim_Block_Adminhtml_System_Config_Fieldset_Info::getKlaviyoVersion() . ' PHP/' . phpversion()
            ));
            $client->setRawData(json_encode($body));
            $response = $client->request();
            return (bool)$response->isSuccessful();
        } catch (Exception $e){
            // Handle php7 zend framework issue with large url's
            $response = file_get_contents($url);
            if (!$response){
                throw new Exception('The following request to Klaviyo failed: ' . $url);
            }
            return $response == '1';
        }
    }
};